from os.path import exists, join
from os import makedirs
import ipdb
import numpy as np
import matplotlib.pyplot as plt
from joblib import dump, load

from sklearn.neighbors import KNeighborsRegressor as KNN
from sklearn.model_selection import GridSearchCV
from sklearn.datasets import make_classification

from config import MIN_RESP_RATE, MAX_RESP_RATE, DATA_DIR

TUNER_TYPE = "gridsearchcv"
DEBUG = False

class KNNClass():
    ''' Class to handle the linear discriminant analysis under gridsearch, must
    have unilateral data classification'''
    def __init__(self, marker=1, directory=None):
        # model parameters
        if DEBUG:
            self.n_neighbors = 3
            self.param_grid = [
                {
                    "p": [1, 2, 3],
                    "leaf_size": [30],
                },
            ]
            self.verbose = 1
        else:
            self.n_neighbors = MAX_RESP_RATE - MIN_RESP_RATE
            self.param_grid = [
                {
                    "p": [1, 2, 3],
                    "leaf_size": [10, 30, 100, 300],
                },
            ]
            self.verbose = 0
            self.scoring = ['neg_mean_absolute_error',
                            'neg_root_mean_squared_error',
                            'r2']
        
        self.is_gridsearch = True
        self.is_regression = True
        self.n_jobs = -1
        if directory is None:
            self.directory = join(DATA_DIR, f'ARIA/knn_{marker}')
        else:
            self.directory = directory
        self.eval_csv = './evaluations/knn.csv'

    def build(self):
        if self.is_regression:
            self.scoring = 'neg_mean_absolute_error'
        else:
            self.scoring = 'f1'

        model = KNN(n_neighbors=self.n_neighbors,
                    n_jobs=self.n_jobs)
        if self.is_gridsearch:
            self.model = GridSearchCV(
                model,
                param_grid=self.param_grid,
                scoring=self.scoring,
                cv=3,
                refit=self.scoring
            )
        else:
            self.model = model
        return self.model

    def save_model(self, filename='knn_model.joblib'):
        # save the model to disk
        if not exists(self.directory): makedirs(self.directory)
        fileout = join(self.directory, filename)
        with open(fileout, 'wb') as mfile:
            dump(self.model, mfile)

    def load_model(self, filename='knn_model.joblib'):
        # load the model from disk
        # self.model = load(filename)
        filein = join(self.directory, filename)
        with open(filein, 'rb') as mfile:
            self.model = load(mfile)

def main():
    # test classification dataset
    print("Making data")
    n_features = 10
    x, y = make_classification(n_samples=1000, n_features=n_features,
                               n_informative=6, n_redundant=4, random_state=1,
                               n_classes=3)
    print('classes: ', np.unique(y))

    # define model
    knn = KNNClass()
    model = knn.build()
    

    # fit model
    print("Fitting data")
    model.fit(x, y)

    # new data
    row = np.random.rand(1, n_features) + np.random.randint(low=-4, high=4,
                                                            size=(1,n_features)
                                                           )
    print("row: ", row)
    print("Predicting")
    yhat = model.predict(row)

    print('Predicted Class: %d' % yhat)

    plt.plot(x[y==0,0], x[y==0,1], 'b.')
    plt.plot(x[y==1,0], x[y==1,1], 'r.')
    plt.plot(x[y==2,0], x[y==2,1], 'k.')
    plt.plot(row[0,0], row[0, 1], 'y^')
    plt.show()

if __name__ == '__main__':
    main()
