import ipdb
from os.path import exists, join
from os import makedirs
import random
import numpy as np
import matplotlib.pyplot as plt
from joblib import dump, load

from sklearn.discriminant_analysis import LinearDiscriminantAnalysis as LDA
from sklearn.model_selection import GridSearchCV
from sklearn.multiclass import OneVsRestClassifier
from sklearn.multioutput import MultiOutputClassifier
from sklearn.datasets import make_classification

MODEL_TYPE = "lda"
TUNER_TYPE = "gridsearchcv"
LDA_DEBUG = True

class LDAClass():
    ''' Class to handle the linear discriminant analysis under gridsearch, must
    have unilateral data classification'''
    def __init__(self, marker=None, directory=None):
        super().__init__()
        # model parameters
        self.param_grid = {
            'estimator__solver': ['svd', 'lsqr', 'eigen'],
            'estimator__tol':    [1e-5, 3e-5, 1e-4, 3e-4]
        }
        self.scoring = ['precision_samples',
                        'recall_samples',
                        'roc_auc_ovr', 
                        'f1']
        if directory is None:
            self.directory = f'/data/rqchia/ARIA/lda_{marker}'
        else:
            self.directory = directory

    def build(self):
        self.model = GridSearchCV(
            OneVsRestClassifier(LDA()),
            param_grid=self.param_grid,
            scoring=self.scoring,
            cv=3,
            refit='f1',
            n_jobs=-1,
        )
        return self.model

    def save_model(self, filename='lda_model.joblib'):
        # save the model to disk
        if not exists(self.directory): makedirs(self.directory)
        fileout = join(self.directory, filename)
        with open(fileout, 'wb') as mfile:
            dump(self.model, mfile)

    def load_model(self, filename='lda_model.joblib'):
        # load the model from disk
        filein = join(self.directory, filename)
        with open(filein, 'rb') as mfile:
            self.model = load(mfile)
        return self.model

def main():
    # test classification dataset
    print("Making data")
    n_features = 10
    x, y = make_classification(n_samples=1000, n_features=n_features,
                               n_informative=6, n_redundant=4, random_state=1,
                               n_classes=3)
    print('classes: ', np.unique(y))

    # define model
    model = LDA()

    # fit model
    print("Fitting data")
    model.fit(x, y)

    # new data
    row = np.random.rand(1, n_features) + np.random.randint(low=-4, high=4,
                                                            size=(1,n_features)
                                                           )
    print("row: ", row)
    print("Predicting")
    yhat = model.predict(row)

    print('Predicted Class: %d' % yhat)

    plt.plot(x[y==0,0], x[y==0,1], 'b.')
    plt.plot(x[y==1,0], x[y==1,1], 'r.')
    plt.plot(x[y==2,0], x[y==2,1], 'k.')
    plt.plot(row[0,0], row[0, 1], 'y^')
    plt.show()

if __name__ == '__main__':
    main()
